#!/usr/bin/env python3
"""
Gridded GLM regression tests
"""

import subprocess
import os
from glob import glob

import netCDF4
import numpy as np

from testlib.common import TESTDIR, MINUTE_GRIDDER
from testlib.common import check_fileglob


def test_issue5_fail1_3():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 1
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))

def test_issue5_fail2_1and3():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 1
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))

def test_issue5_fail2_2and3():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 1
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))

def test_issue5_3minutes():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740001000_e20220740001200_c20220740001225.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740001200_e20220740001400_c20220740001419.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740001400_e20220740002000_c20220740002019.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002000_e20220740002200_c20220740002228.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002200_e20220740002400_c20220740002427.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002400_e20220740003000_c20220740003018.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16_s20220740000000_e20220740001000_c*.nc"))
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16_s20220740001000_e20220740002000_c*.nc"))
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16_s20220740002000_e20220740003000_c*.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T04_20220315000100.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T04_20220315000200.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T04_20220315000300.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T05_20220315000100.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T05_20220315000200.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T05_20220315000300.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T07_20220315000100.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T07_20220315000200.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T07_20220315000300.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000100.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000200.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000300.nc"))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_statistics_*.nc", 3))

def test_issue5_crossminutes_fail():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740001200_e20220740001400_c20220740001419.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002400_e20220740003000_c20220740003018.nc"),
                                        ])
    assert completed_process.returncode == 1
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))

def test_issue5_realtime_early1():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        "--realtime",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        ])
    # we expect people using realtime mode to run this on every file, so a return code of 0 is expected on a file before the end of the minute
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))

def test_issue5_realtime_early2():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        "--realtime",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740001200_e20220740001400_c20220740001419.nc"),
                                        ])
    # we expect people using realtime mode to run this on every file, so a return code of 0 is expected on a file before the end of the minute
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-*.nc", 0))
    assert(check_fileglob("CSPP_OR_GLM-*.nc", 0))


# test for providing two minutes with a minute in between that has no input
# the middle minute should produce no output
def test_issue7():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002000_e20220740002200_c20220740002228.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002200_e20220740002400_c20220740002427.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002400_e20220740003000_c20220740003018.nc"),
                                        ])
    assert completed_process.returncode == 0
    # check for 2 output gridded files
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16*.nc", 2))
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16_s20220740002000_e20220740003000_c*", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000300.nc", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T05_20220315000300.nc", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T04_20220315000300.nc", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T07_20220315000300.nc", 1))

# test for proper handling of negative center lat/lons
def test_issue22():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMM1-M3_G16_s20220740000000_e20220740001000_c*.nc", 1))

def test_issue23_oor_lon_high():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "200",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode != 0
    assert(check_fileglob("CG_GLM-L2-*.nc", 0))

def test_issue23_oor_lon_low():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "-181",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode != 0
    assert(check_fileglob("CG_GLM-L2-*.nc", 0))

def test_issue23_oor_lat_high():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "91",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode != 0
    assert(check_fileglob("CG_GLM-L2-*.nc", 0))

def test_issue23_oor_lat_low():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "-90.30",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode != 0
    assert(check_fileglob("CG_GLM-L2-*.nc", 0))

def test_issue24_meso1():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso1",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMM1-M3_G16_s20220740000000_e20220740001000_c*.nc", 1))

def test_issue24_meso2():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso2",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMM2-M3_G16_s20220740000000_e20220740001000_c*.nc", 1))

def test_issue24_meso2_tiles():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "32.05",
                                        "--ctr-lon", "-96.85",
                                        "--goes-sector", "meso2",
                                        "--create-tiles",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMM2-M3_G16_s20220740000000_e20220740001000_c*.nc", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMM2-M3_G16_T01_20220315000100.nc", 1))

# test that we still produce a file when there's no valid data inside the gridded region
def test_issue38_no_valid_data():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--ctr-lat", "43.0722",
                                        "--ctr-lon", "-89.4008",
                                        "--goes-sector", "meso",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000000_e20220740000200_c20220740000227.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000200_e20220740000400_c20220740000428.nc"),
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740000400_e20220740001000_c20220740001027.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMM1-M3_G16_s20220740000000_e20220740001000_c*.nc", 1))

    # check that the file contents are blank too
    mesofilename = glob("CG_GLM-L2-GLMM1-M3_G16_s20220740000000_e20220740001000_c*.nc")[0]
    ds = netCDF4.Dataset(mesofilename, 'r')
    assert(ds.variables['minimum_flash_area'][:].all() is np.ma.masked)
    assert(ds.variables['total_energy'][:].all() is np.ma.masked)
    assert(ds.variables['total_energy'][:].all() is np.ma.masked)
    assert((ds.variables['DQF'][:] == 0).all())
