#!/usr/bin/env python3
"""
Gridded GLM tile testing
"""

import subprocess
import os
import warnings
import netCDF4

from testlib.common import TESTDIR, MINUTE_GRIDDER
from testlib.common import check_fileglob

"""
Check our tile variable names against a sample metadata listing for a tile the ground segment generated.

Provided by Lee Byerle via email on 2022-09-21
(DR_GLMFD-020-B16-M0C00-T021_G18_s2022255171000_c2022255171140)
"""
def test_tile_variable_names():
    completed_process = subprocess.run(['python',
                                        MINUTE_GRIDDER,
                                        "--goes-sector", "conus",
                                        "--create-tiles",
                                        "--realtime",
                                        os.path.join(TESTDIR, "test-inputs/OR_GLM-L2-LCFA_G16_s20220740002400_e20220740003000_c20220740003018.nc"),
                                        ])
    assert completed_process.returncode == 0
    assert(check_fileglob("CG_GLM-L2-GLMC-M3_G16_s20220740002000_e20220740003000_c*", 1))
    assert(check_fileglob("CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000300.nc", 1))
    ds = netCDF4.Dataset('CSPP_OR_GLM-L2-GLMC-M3_G16_T08_20220315000300.nc', 'r')
    assert('x' in ds.variables)
    assert('y' in ds.variables)
    assert('Flash_extent_density' in ds.variables)
    assert('Total_Optical_energy' in ds.variables)
    assert('Minimum_flash_area' in ds.variables)
    assert('DQF' in ds.variables)